import culqipy
import unittest
import uuid

culqipy.public_key = 'pk_test_vzMuTHoueOMlgUPj'
culqipy.secret_key = 'sk_test_UTCQSGcXW8bCyU59'


class TestStringMethods(unittest.TestCase):
    def token(self):

        dir_token = {'card_number': '4111111111111111',
                     'cvv': '123',
                     'currency_code': 'PEN',
                     'email': 'wmuro@me.com',
                     'expiration_month': 9,
                     'expiration_year': 2020}

        return culqipy.Token.create(dir_token)

    def charge(self):
        dir_charge = {'amount': 1000,
                      'capture': True,
                      'currency_code': 'PEN',
                      'description': 'Venta de prueba',
                      'email': 'wmuro@me.com',
                      'installments': 0,
                      'metadata': {'test': '1234'},
                      'source_id': self.token()["id"]}
        return culqipy.Charge.create(dir_charge)

    def plan(self):
        dir_plan = {'amount': 1000,
                    'currency_code': 'PEN',
                    'interval': 'dias',
                    'interval_count': 2,
                    'limit': 10,
                    'metadata': {'test': '1234'},
                    'name': 'plan-test-' + str(uuid.uuid1()),
                    'trial_days': 50}
        return culqipy.Plan.create(dir_plan)

    def customer(self):
        dir_customer = {'address': 'Avenida Lima 123213',
                        'address_city': 'LIMA',
                        'country_code': 'PE',
                        'email': 'wmuro'+str(uuid.uuid1())+'@me.com',
                        'first_name': 'William',
                        'last_name': 'Muro',
                        'metadata': {'other_email': 'wam@yahoo.com'},
                        'phone_number': 998989789,
                        }
        return culqipy.Customer.create(dir_customer)

    def card(self):
        dir_card = {'customer_id': self.customer()["id"],
                   'token_id': self.token()["id"]}
        return culqipy.Card.create(dir_card)

    def subscription(self):
        dir_subscription = {'card_id': self.card()["id"],
                            'plan_id': self.plan()["id"]}

        subscription = culqipy.Subscription.create(dir_subscription)
        return subscription

    #def refund(self):
    #    dir_refund = {'amount': 500, 'charge_id': self.charge()["id"], 'reason': 'give me money back'}
    #    refund = culqipy.Refund.create(dir_refund)
    #    return refund

    def test_0_list_tokens(self):
        params = {'iin': '411111'}
        token_list = culqipy.Token.list(params)
        data = False
        if len(token_list) > 0:
            data = True
        self.assertTrue(data)

    def test_1_token(self):
        self.assertEqual("token", str(self.token()["object"]))

    def test_2_find_token(self):
        id = self.token()["id"]
        token = culqipy.Token.get(id)
        self.assertEqual("token", str(token["object"]))

    def test_3_charge(self):
        self.assertEqual("charge", str(self.charge()["object"]))

    def test_4_charge_capture(self):
        capture_charge = culqipy.Charge.capture( str(self.charge()["id"]) )
        self.assertNotEqual("charge", str(capture_charge["object"]))

    def test_5_list_charge(self):
        params = {'min_amount': 500, 'max_amount': 1000}
        charge_list = culqipy.Charge.list(params)
        data = False
        if len(charge_list) > 0:
            data = True
        self.assertTrue(data)

    def test_6_plan(self):
        self.assertEqual("plan", str(self.plan()["object"]))

    def test_7_customer(self):
        self.assertEqual("customer", str(self.customer()["object"]))

    def test_8_card(self):
        self.assertEqual("card", str(self.card()["object"]))

    def test_9_subscription(self):
        self.assertEqual("subscription", str(self.subscription()["object"]))

    #def test_refund(self):
    #   self.assertEqual("refund", str(self.refund()["object"]))


if __name__ == '__main__':
    unittest.main()



import culqipy
import requests
import json


class Util:
    @staticmethod
    def json_result(key, url, data, method):
        timeout = 60
        if method.upper() == "GET":
            timeout = 360

        headers = {
            "Authorization": "Bearer " + key,
            "content-type": "application/json"
        }
        r = ""
        try:
            if method.upper() == "GET":
                if not data:
                    r = requests.get(
                        culqipy.API_URL + url,
                        headers=headers,
                        timeout=timeout
                    )
                else:
                    r = requests.get(
                        culqipy.API_URL + url,
                        headers=headers,
                        params=json.dumps(data),
                        timeout=timeout
                    )
            if method.upper() == "POST":
                r = requests.post(
                        culqipy.API_URL + url,
                        headers=headers,
                        data=json.dumps(data),
                        timeout=timeout
                    )
            if method.upper() == "DELETE":
                r = requests.delete(
                        culqipy.API_URL + url,
                        headers=headers,
                        timeout=timeout
                    )
            if method.upper() == "PATCH":
                r = requests.patch(
                        culqipy.API_URL + url,
                        headers=headers,
                        data=json.dumps(data),
                        timeout=timeout
                    )
            return r
        except requests.exceptions.RequestException:
            error = {"object": "error", "type": "server",
                     "code": "404",
                     "message": "connection...",
                     "user_message": "Connection Error!"}
            return json.dumps(error)


class CulqiError(Exception):
    def __init__(self, response_error):
        self.response_error = response_error


class Operation():
    @staticmethod
    def list(url, api_key, params):
        try:
            response = Util().json_result(
                    api_key,
                    url,
                    params, "GET")
            if response.status_code == "401":
                raise CulqiError(response.json())
            if response.status_code == "403":
                raise CulqiError(response.json())
            if response.status_code == "404":
                raise CulqiError(response.json())
            return response.json()
        except CulqiError as ce:
            return ce.response_error

    @staticmethod
    def create(url, api_key, body):
        try:
            response = Util().json_result(
                api_key,
                url,
                body, "POST")
            if response.json()["object"] == "error":
                raise CulqiError(response.json())
            return response.json()
        except CulqiError as ce:
            return ce.response_error

    @staticmethod
    def get_delete(url, api_key, id, method):
        try:
            response = Util().json_result(
                    api_key,
                    url + id + "/",
                    "", method)
            if response.json()["object"] == "error":
                raise CulqiError(response.json())
            return response.json()
        except CulqiError as ce:
            return ce.response_error

    @staticmethod
    def update(url, api_key, id, body):
        try:
            response = Util().json_result(
                api_key,
                url + id + "/",
                body, "PATCH")
            if response.json()["object"] == "error":
                raise CulqiError(response.json())
            return response.json()
        except CulqiError as ce:
            return ce.response_error


class Card:
    URL = "/cards/"

    @staticmethod
    def create(body):
        return Operation.create(Card.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def delete(id):
        return Operation.get_delete(Card.URL,
                                    culqipy.secret_key, id, "DELETE")

    @staticmethod
    def get(id):
        return Operation.get_delete(Card.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def update(id, body):
        return Operation.update(Card.URL,
                                    culqipy.secret_key, id, body)


Card = Card()


class Event:
    URL = "/events/"

    @staticmethod
    def list(params):
        return Operation.list(Event.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def get(id):
        return Operation.get_delete(Event.URL,
                                    culqipy.secret_key, id, "GET")


Event = Event()


class Customer:
    URL = "/customers/"

    @staticmethod
    def list(params):
        return Operation.list(Customer.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Customer.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def delete(id):
        return Operation.get_delete(Customer.URL,
                                    culqipy.secret_key, id, "DELETE")

    @staticmethod
    def get(id):
        return Operation.get_delete(Customer.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def update(id, body):
        return Operation.update(Customer.URL,
                                culqipy.secret_key, id, body)


Customer = Customer()


class Transfer:
    URL = "/transfers/"

    @staticmethod
    def list(params):
        return Operation.list(Transfer.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def get(id):
        return Operation.get_delete(Transfer.URL,
                                    culqipy.secret_key, id, "GET")


Transfer = Transfer()


class Iins:
    URL = "/iins/"

    @staticmethod
    def list(params):
        return Operation.list(Iins.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def get(id):
        return Operation.get_delete(Iins.URL,
                                    culqipy.secret_key, id, "GET")


Iins = Iins()


class Token:
    URL = "/tokens/"

    @staticmethod
    def list(params):
        return Operation.list(Token.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Token.URL,
                                culqipy.public_key, body)

    @staticmethod
    def get(id):
        return Operation.get_delete(Token.URL,
                                    culqipy.secret_key, id, "GET")


Token = Token()


class Charge:
    URL = "/charges/"

    @staticmethod
    def list(params):
        return Operation.list(Charge.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Charge.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def get(id):
        return Operation.get_delete(Charge.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def capture(id):
        try:
            response = Util().json_result(
                culqipy.secret_key,
                Charge.URL + id + "/capture/",
                "", "POST")
            if response.json()["object"] == "error":
                raise CulqiError(response.json())
            return response.json()
        except CulqiError as ce:
            return ce.response_error

    @staticmethod
    def update(id, body):
        return Operation.update(Charge.URL,
                                culqipy.secret_key, id, body)


Charge = Charge()


class Plan:
    URL = "/plans/"

    @staticmethod
    def list(params):
        return Operation.list(Plan.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Plan.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def delete(id):
        return Operation.get_delete(Plan.URL,
                                    culqipy.secret_key, id, "DELETE")

    @staticmethod
    def get(id):
        return Operation.get_delete(Plan.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def update(id, body):
        return Operation.update(Plan.URL,
                                culqipy.secret_key, id, body)


Plan = Plan()


class Subscription:
    URL = "/subscriptions/"

    @staticmethod
    def list(params):
        return Operation.list(Subscription.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Subscription.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def delete(id):
        return Operation.get_delete(Subscription.URL,
                                    culqipy.secret_key, id, "DELETE")

    @staticmethod
    def get(id):
        return Operation.get_delete(Subscription.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def update(id, body):
        return Operation.update(Subscription.URL,
                                culqipy.secret_key, id, body)


Subscription = Subscription()


class Refund:
    URL = "/refunds/"

    @staticmethod
    def list(params):
        return Operation.list(Refund.URL,
                              culqipy.secret_key, params)

    @staticmethod
    def create(body):
        return Operation.create(Refund.URL,
                                culqipy.secret_key, body)

    @staticmethod
    def get(id):
        return Operation.get_delete(Refund.URL,
                                    culqipy.secret_key, id, "GET")

    @staticmethod
    def update(id, body):
        return Operation.update(Refund.URL,
                                culqipy.secret_key, id, body)


Refund = Refund()



# Resource

from culqipy.resource import (
    Token,
    Charge,
    Plan,
    Subscription,
    Refund,
    Iins,
    Card,
    Event,
    Customer,
    Transfer
)

# Configuration variables

API_URL = "https://api.culqi.com/v2"
public_key = None
secret_key = None
